Ext.ns('app.confluence');

app.confluence.Proxy = {
	/**
	 * List all spaces
	 * @param {Object} config An object with the following properties
	 * 	success - called if search is successful
	 *  fail - called if search fails with an error
	 *  scope - a scope for callbacks
	 */
	listSpaces: function(config) {
		var rpc = function(token) {
			app.xmlrpc.XmlRpc.exec({
				url: app.getRpcUrl(),
				method: 'confluence2.getSpaces', 
				params: [app.token], 
				success: config.success, 
				fail: config.fail, 
				scope: config.scope
			});
		};

		app.login({
			success: rpc, 
			fail: config.fail
		});
	}, 
	
	/**
	 * Lists all pages in a given space
	 * @param {Object} config An object with the following properties
	 *  spaceKey - key identifying the space to load
	 * 	success - called if search is successful
	 *  fail - called if search fails with an error
	 *  scope - a scope for callbacks
	 */
	listPages: function(config) {
		var rpc = function(token) {
			app.xmlrpc.XmlRpc.exec({
				url: app.getRpcUrl(),
				method: 'confluence2.getPages', 
				params: [app.token, config.spaceKey], 
				success: config.success, 
				fail: config.fail, 
				scope: config.scope
			});
		};

		app.login({
			success: rpc, 
			fail: config.fail
		});
	}, 
	
	/**
	 * Gets a single page
	 * @param {Object} config An object with the following properties
	 *  pageId - the id of the page to get
	 * 	success - called if search is successful
	 *  fail - called if search fails with an error
	 *  scope - a scope for callbacks
	 */
	getPage: function(config) {
		var rpc = function(token) {
			app.xmlrpc.XmlRpc.exec({
				url: app.getRpcUrl(),
				method: 'confluence2.getPage', 
				params: [app.token, config.pageId], 
				success: config.success, 
				fail: config.fail, 
				scope: config.scope
			});
		};

		app.login({
			success: rpc, 
			fail: config.fail
		});
	}, 
	
	/**
	 * Gets a list of attachments for a page
	 * @param {Object} config An object with the following properties
	 *  pageId - the id of the page
	 * 	success - called if search is successful
	 *  fail - called if search fails with an error
	 *  scope - a scope for callbacks
	 */
	getAttachments: function(config) {
		var rpc = function(token) {
			app.xmlrpc.XmlRpc.exec({
				url: app.getRpcUrl(),
				method: 'confluence2.getAttachments', 
				params: [app.token, config.pageId], 
				success: config.success, 
				fail: config.fail, 
				scope: config.scope
			});
		};

		app.login({
			success: rpc, 
			fail: config.fail
		});
	}, 
	
	createPage: function() {
		
	}, 
	
	/**
	 * @param {Object} config an object with the following properties:
	 *  pageId - the id of the page to add the attachment to
	 *  attachment - An object with the following properties:
	 *  	fileName - The name of the attachment
	 *  	fileSize - The size of the file
	 *  	contentType - The mime type
	 *  	comment - A comment for the attachment
	 *  data - the photo encoded as a base64 string
	 * 	success - called if search is successful
	 *  fail - called if search fails with an error
	 *  scope - a scope for callbacks
	 */
	addAttachment: function(config) {
		app.xmlrpc.XmlRpc.exec({
			url: app.getRpcUrl(), 
			method: 'confluence2.addAttachment', 
			params: [app.token, config.pageId, config.attachment, new app.xmlrpc.Base64Value({value: config.data})], 
			success: config.success, 
			fail: config.fail, 
			scope: config.scope
		});
	}, 
	
	/**
	 * Creates a new page
	 * @param config
	 * @param {Object} config an object with the following properties:
	 *  space - The key representing the space in which the page will be placed.
	 *  attachment - An object with the following properties:
	 *  	fileName - The name of the attachment
	 *  	fileSize - The size of the file
	 *  	contentType - The mime type
	 *  	comment - A comment for the attachment
	 *  data - the photo encoded as a base64 string
	 * 	success - called if search is successful
	 *  fail - called if search fails with an error
	 *  scope - a scope for callbacks
	 */
	addPageWithAttachment: function(config) {
		var success = function(savedPage) {
			config.pageId = savedPage.id;
			app.confluence.Proxy.addAttachment(config);
		};
		
		var page = {
			space: config.space, 
			title: config.title,
			content: config.attachment.comment
		};
		
		app.xmlrpc.XmlRpc.exec({
			url: app.getRpcUrl(), 
			method: 'confluence2.storePage', 
			params: [app.token, page], 
			success: success, 
			fail: config.fail, 
			scope: config.scope
		});
	}, 
	
	/**
	 * Deletes an attachment.
	 * @param config
	 * @param {Object} config an object with the following properties:
	 *  pageId - The id of the page to remove the attachment from
	 *  fileName - The name of the attachment to remove
	 * 	success - called if search is successful
	 *  fail - called if search fails with an error
	 *  scope - a scope for callbacks
	 */
	removeAttachment: function(config) {
		app.xmlrpc.XmlRpc.exec({
			url: app.getRpcUrl(), 
			method: 'confluence2.removeAttachment', 
			params: [app.token, config.pageId, config.fileName], 
			success: config.success, 
			fail: config.fail, 
			scope: config.scope
		});
	}, 

	/**
	 * Deletes a page.
	 * @param config
	 * @param {Object} config an object with the following properties:
	 *  pageId - The id of the page to remove the attachment from
	 * 	success - called if search is successful
	 *  fail - called if search fails with an error
	 *  scope - a scope for callbacks
	 */
	removePage: function(config) {
		app.xmlrpc.XmlRpc.exec({
			url: app.getRpcUrl(), 
			method: 'confluence2.removePage', 
			params: [app.token, config.pageId], 
			success: config.success, 
			fail: config.fail, 
			scope: config.scope
		});
	}
};
