app.views.CameraForm = Ext.extend(Ext.form.FormPanel, {
	bodyStyle: 'margin:0; padding: 0', 
	defaults: {
		labelWidth: 95
	}, 

	initComponent: function() {
		this.textField = new Ext.form.TextArea({
			label: 'Comment:<div style="font-size:70%">(optional)</div>',
			placeHolder: 'Enter a description'
		});
		
		this.nameField = new Ext.form.Text({
			style: 'background-color:white', 
			label: 'Title:',
			hidden: true, 
			placeHolder: 'Name your page'
		});
		
		this.image = {
			xtype: 'field',
			text: 'Use Camera', 
			label: 'Photo:', 
			style: {
				'background-color': 'white'
			},
			listeners: {
				afterrender: this.renderCameraButton, 
				scope: this
			}
		};
		
		this.items = [this.nameField, this.image, this.textField];
		
		this.uploadButton = new Ext.Button({
			xtype: 'button',
			// disabled: true, 
			ui: 'confirm-round', 
			handler: this.uploadPhoto, 
			scope: this, 
			text: 'Upload Photo'
		});
		
		this.dockedItems = [{
			xtype: 'toolbar', 
			dock: 'bottom', 
			layout: {
				type: 'hbox', 
				pack: 'center'
			}, 
			items: [this.uploadButton]
		}];
		
		this.addEvents(
			/**
			 * @event Save
			 * Called after an attachment is uploaded or a new page is created.
			 * @param {app.views.CameraForm} this
			 * @param {String} pageId The id of the page that was created or updated.
			 */
			'save'
		);
		
		app.views.CameraForm.superclass.initComponent.apply(this, arguments);
	}, 
	
	/**
	 * Renders the camera button and image preview placeholder.
	 * @param {Ext.form.Field} cmp The placeholder field where the camera button is meant to be rendered
	 */
	renderCameraButton: function(cmp) {
		cmp.el.down('.x-form-field-container').update('<div id="preview-wrap"></div>');
		
		this.cameraButton = new Ext.Button({
			width: 185, 
			ui: 'confirm-round', 
			iconCls: 'camera-icon', 
			text: 'Use Camera', 
			handler: this.useCamera, 
			scope: this
		});
		
		this.preview = new Ext.Panel({
			hidden: true, 
			bodyStyle: 'background-color:transparent; border: none; padding: 0 0 5px 0;', 
			html: '<img id="preview" height="100" src="resources/images/example-photo.jpeg"/>'
		});
		
		new Ext.Panel({
			height: 170, 
			bodyStyle: 'background-color:transparent; padding: 10px; border: none;', 
			renderTo: 'preview-wrap', 
			items: [this.preview, this.cameraButton]
		});
	}, 
	
	/**
	 * Activates the camera via PhoneGap
	 */
	useCamera: function() {
		var me = this;
		
		var success = function(imageData) {
			me.onPhotoSuccess(imageData);
		};
		
		var fail = function() {
			me.onPhotoFail();
		};
		
		navigator.camera.getPicture(success, fail, { 
			quality: parseInt(app.getSetting('photoQuality') || '50'), 
			//allowEdit: true, 
			correctOrientation: true
		}); 
	},
	
	/**
	 * PhoneGap camera success callback
	 * @param {String} imageData The image encoded as a base64 string.
	 */
	onPhotoSuccess: function(imageData) {
		this.cameraButton.setText('Retake');
		this.imageData = imageData;
		this.preview.show();
		this.uploadButton.setDisabled(false);
		Ext.fly('preview').dom.src = "data:image/jpeg;base64," + this.imageData;
	},
	
	/**
	 * Resets the form, clearing the image, page name, and text
	 */
	reset: function() {
		this.nameField.setValue('');
		this.textField.setValue('');
		this.preview.hide();
	}, 
	
	/**
	 * Uploads the photo to Confluence.
	 * @param page
	 */
	uploadPhoto: function() {
		var fn = app.confluence.Proxy.addPageWithAttachment;

		if(!this.isValid()) {
			return;
		}
		
		if(this.page) {
			fn = app.confluence.Proxy.addAttachment;
		}

		var attachment = {
			fileName: 'iPhone upload ' + new Date().getTime() + '.jpeg', 
			fileSize: this.imageData ? this.imageData.length : 0, 
			contentType: 'image/jpeg', 
			comment: this.textField.getValue() 
		};
		
		var config = {
			space: this.spaceKey, 
			pageId: this.page ? this.page.id: undefined, 
			data: this.imageData, 
			title: this.nameField.getValue(), 
			attachment: attachment, 
			scope: this, 
			fail: this.onUploadFail, 
			success: this.onUploadSuccess, 
		};
		fn.call(app.confluence.Proxy, config);
	}, 
	
	onUploadSuccess: function(attachment) {
		this.cameraButton.setText('Use Camera');
		this.uploadButton.setDisabled(true);
		this.reset();
		this.fireEvent('save', this, attachment.pageId);
	}, 
	
	onUploadFail: function(message) {
		Ext.Msg.alert('Error', message || 'An unkown error occurred.');
	}, 
	
	/**
	 * Validates the form.  Makes sure Title and Photo fields are filled out.
	 * @returns {Boolean} true if valid, otherwise false
	 */
	isValid: function() {
		var invalidFields = [];

		if(!this.page && !this.nameField.getValue()) {
			invalidFields.push('Please specify a title.');
		}
		
		if(!this.imageData) {
			//invalidFields.push('Please tap "Use Camera" to take a photo.');
		}
		
		if(invalidFields.length) {
			Ext.Msg.alert(null, invalidFields.join('<br/>'));
			return false;
		} else {
			return true;
		}
	}, 
	
	/**
	 * Sets the page being uploaded to and toggles.  Shows the name field if no page is specified, otherwise
	 * hides the name field.
	 * @param {Object} A confluence page object.
	 */
	setPage: function(page) {
		this.page = page;
		
		if(!this.page) {
			this.nameField.show();
		} else {
			this.nameField.hide();
		}
	}, 
	
	/**
	 * For new pages, sets the key representing the space being added to.
	 * @param {String} spaceKey The key representing a confluence space.
	 */
	setSpaceKey: function(spaceKey) {
		this.spaceKey = spaceKey;
	}
});
