Ext.ns('app.views');

app.views.Page = Ext.extend(Ext.TabPanel, {
	tabBar: {
		dock: 'bottom',
		layout: {
			pack: 'center'
		}
	}, 
	
	initComponent: function() {
		this.createStore();
		this.createItems();
		
		this.addEvents(
			/**
			 * @event delete
			 * Fired when the current page is deleted.
			 */
			'delete', 
			
			/**
			 * @event load
			 * Fired when a page loads
			 */
			'load'
		);
		
		app.views.Page.superclass.initComponent.apply(this, arguments);
	}, 

	createStore: function() {
		this.attachmentsStore = new Ext.data.Store({
			model: 'app.models.Attachment', 
			listeners: {
				scope: this, 
				attachmentsList: function() {
					this.spaceList.scroller.scrollTo({
					    x: 0,
					    y: 0
					});
				}
			}
		});
	}, 
	
	createItems: function() {
		this.contentPanel = new Ext.Panel({
			title: 'Page', 
			iconCls: 'page-icon', 
			dockedItems: this.contentToolbar,
			scroll: 'vertical', 
			cls: 'page-content',
			baseCls: 'page-tab', 
			flex: 1 
		});

		this.attachmentsList = new Ext.List({
			store: this.attachmentsStore, 
			bodyStyle: 'background-color:white', 
			itemTpl: '<div class="attachment"><div>{title}</div><div>{comment}</div></div>',
			onItemDisclosure: this.openAttachmentInSafari, 
			listeners: {
				scope: this, 
				selectionchange: function() {
					this.toggleDeleteDisabled();
				}
			}
		});
		
		this.attachmentsPanel = new Ext.Panel({
			title: 'Attachments', 
			iconCls: 'attachments-icon', 
			dockedItems: this.attachmentsToolbar, 
			height: 42, 
			layout: 'card',
			activeItem: 0, 
			items: [this.attachmentsList, {padding: 10, html: '<div class="no-attachments-message">There are no attachments on this page.</div>'}], 
			listeners: {
				scope: this, 
				show: function() {
					this.toggleDeleteDisabled(this.attachmentsPanel); 
				}, 
				hide: function() {
					this.toggleDeleteDisabled(this.contentPanel);
				}
			}
		});

		/**
		 * @property {Ext.Button} addPhotoButton
		 * The button that when clicked should bring up the camera form.
		 */
		this.addPhotoButton = new Ext.Button({
			xtype: 'button',
			ui: 'plain', 
			iconMask: true, 
			iconCls: 'camera-toolbar-icon', 
		});
		
		this.deleteButton = new Ext.Button({
			xtype: 'button', 
			iconMask: true, 
			ui: 'plain', 
			iconCls: 'trash', 
			handler: this.onDeleteTap, 
			scope: this
		});
		
		this.dockedItems = {
			xtype: 'toolbar',
			ui: 'plain', 
			layout: {
				type: 'hbox'
			}, 
			items: [this.addPhotoButton, {
				xtype: 'spacer', 
				flex: 1
			}, {
				xtype: 'button', 
				iconCls: 'refresh',
				ui: 'plain', 
				iconMask: true, 
				handler: this.refresh, 
				scope: this
			}, this.deleteButton, {
				xtype: 'button', 
				iconMask: true, 
				ui: 'plain', 
				iconCls: 'action', 
				handler: this.openPageInSafari, 
				scope: this
			}]
			
		}, 
		
		this.items = [this.contentPanel, this.attachmentsPanel];
	}, 
	
	/**
	 * Reloads the current page.
	 */
	refresh: function() {
		this.loadPage(this.page.id, true);
	}, 
	
	/**
	 * Loads a page by id
	 * @param {String} id The id of the page to load.
	 * @param {Boolean} if false, the page tab will become selected
	 */
	loadPage: function(id, keepCurrentTab) {
		if(!keepCurrentTab) {
			this.layout.setActiveItem(0);
		}
		app.confluence.Proxy.getPage({
			pageId: id, 
			success: this.setPage,
			scope: this
		});
	}, 
	
	/**
	 * Sets title, content, etc based on the specified page
	 * @param {Object} page A confluence Page object.  See https://developer.atlassian.com/display/CONFDEV/Confluence+XML-RPC+and+SOAP+APIs
	 */
	setPage: function(page) {
		this.page = page;
		this.contentPanel.update(page.content);
		
		app.confluence.Proxy.getAttachments({
			pageId: page.id, 
			success: this.setAttachments,
			scope: this
		});
	}, 
	
	/**
	 * Loads the specified attachments into the attachments store
	 * @param {Array} attachments
	 */
	setAttachments: function(attachments) {
		this.attachmentsPanel.layout.setActiveItem(attachments.length ? 0 : 1);
		this.getTabBar().items.getAt(1).setBadge(attachments.length ? attachments.length : null);
		this.attachmentsStore.loadData(attachments);
		this.doLayout();
		this.fireEvent('load', this, this.page);
	}, 

	/**
	 * Called when the disclosure button is tapped.  Opens the selected attachment in Safari.
	 * @param {Object} attachment
	 * @param {Ext.Button} btn
	 * @param {Number} index
	 */
	openAttachmentInSafari: function(attachment, btn, index) {
		/*
		 * Since we have configured all non-local urls to open in Safari, the following will
		 * open the attachment in Safari, not Camcon.
		 */
		
		Ext.Msg.confirm(null, 'View attachment in Safari?', function(response) {
			if(response == 'yes') {
				self.location.href = attachment.get('url');
			}
		});
	},
	
	/**
	 * Opens the current page in Safari.
	 */
	openPageInSafari: function() {
		/*
		 * Since we have configured all non-local urls to open in Safari, the following will
		 * open the attachment in Safari, not Camcon.
		 */
		Ext.Msg.confirm(null, 'View this page in Safari?', function(response) {
			if(response == 'yes') {
				self.location.href = this.page.url;
			}
		}, this);
	}, 
	
	/**
	 * Called when the delete button is tapped.
	 */
	onDeleteTap: function() {
		if(this.layout.getActiveItem() === this.contentPanel) {
			this.deletePage();
		} else {
			this.deleteAttachment();
		}
	}, 
	
	/**
	 * Deletes the current page.
	 */
	deletePage: function() {
		Ext.Msg.confirm(null, 'Delete this page?', function(response) {
			if(response == 'yes') {
				app.confluence.Proxy.removePage({
					pageId: this.page.id,
					success: function() {
						this.fireEvent('delete', this);
					}, 
					scope: this
				});
			}
		}, this);
	}, 
	
	/**
	 * Deletes the currently selected attachment.
	 */
	deleteAttachment: function() {
		var attachment = this.attachmentsList.getSelectedRecords()[0];
		
		if(!attachment) {
			Ext.Msg.alert(null, 'Please select an attachment to delete');
			return;
		};
		
		Ext.Msg.confirm(null, 'Delete this attachment?', function(response) {
			if(response == 'yes') {
				app.confluence.Proxy.removeAttachment({
					pageId: attachment.get('pageId'),
					fileName: attachment.get('fileName'), 
					success: this.refresh, 
					scope: this
				});
			}
		}, this);
	}, 
	
	/**
	 * Enables/disables the delete button.  The delete button will be enabled if and only if the page tab 
	 * is selected or the attachments tab is selected and an attachment is selected.
	 */
	toggleDeleteDisabled: function(activeTab) {
		activeTab = activeTab || this.layout.getActiveItem();
		this.deleteButton.setDisabled(activeTab === this.attachmentsPanel && this.attachmentsList.getSelectionCount() == 0);
	}
});
