/**
 * This class represents the main view for the app, which consists of three cards:
 * 1.) A list of spaces 
 * 2.) A list of pages, filtered by the space selection
 * 3.) A form to fill out that allows the user to access the camera.
 */
app.views.SpaceBrowser = Ext.extend(Ext.Panel, {
	layout: 'card', 
	activeItem: 0, 
	
	initComponent: function() {
		this.backButton = new Ext.Button({
			ui: 'back', 
			text: 'Back', 
			hidden: true, 
			handler: this.goBack, 
			scope: this
		});

		this.toolbar = new Ext.Toolbar({
			ui: 'light', 
			cls: 'browser-toolbar', 
			dock: 'top', 
			title: 'All Spaces', 
			items: [this.backButton, {xtype: 'spacer', flex: 1}, {
        		xtype: 'button', 
        		iconCls: 'settings',
        		iconMask: true, 
        		ui: 'plain', 
        		handler: app.showSettings, 
        		scope: app
        	}]
		});
		
		this.dockedItems = [this.toolbar];
		this.createStores();
		this.items = [this.createSpacePanel(), this.createPagePanel(), this.createPageView(), this.createCameraForm()];
		this.on('afterrender', this.listSpaces, this);
		
		app.views.SpaceBrowser.superclass.initComponent.apply(this, arguments);
	}, 
	
	/**
	 * Creates the camera form.
	 * @returns {app.views.CameraForm}
	 */
	createCameraForm: function() {
		this.cameraForm = new app.views.CameraForm({
			listeners: {
				scope: this, 
				show: function() {
					this.toolbar.setTitle(!this.currentPage ? 'New Page' : this.truncateTitle(this.currentPage.title));
				}, 
				save: function(cameraForm, pageId) {
					this.layout.setActiveItem(2, {
						type: 'slide',
						reveal: true, 
						direction: 'down'
					});
					
					this.pageView.loadPage(pageId, true);
					this.pageView.layout.setActiveItem(1);
				}
			}
		});
		
		return this.cameraForm;
	}, 
	
	createPageView: function() {
		this.pageView = new app.views.Page({
			listeners: {
				scope: this, 
				load: function(view, page) {
					this.toolbar.setTitle(this.truncateTitle(page.title));
					this.currentPage = page;
				}, 
				'delete': function() {
					this.goBack();
					this.refresh();
				}
			}
		});
		this.pageView.addPhotoButton.handler = this.showCameraForm;
		this.pageView.addPhotoButton.scope = this;
		return this.pageView;
	}, 
	
	/**
	 * Truncates a string with "..." so that it can fit in the title of a toolbar.
	 * @param {String} value the title to truncate
	 * @returns {String} the truncated string
	 */
	truncateTitle: function(value) {
		return Ext.util.Format.ellipsis(value, 25, false);
	}, 
	
	/**
	 * Creates the space list and search field.
	 * @returns {Ext.FormPanel}
	 */
	createSpacePanel: function() {
		this.spaceList = new Ext.List({
			store: this.spaceStore, 
			itemTpl: '<div class="page"><div>{title}</div></div>', 
			grouped: true, 
			indexBar: true, 
			listeners: {
				scope: this, 
				itemtap: this.onSpaceTap
			}
		});
		
		this.spaceSearchField = new Ext.form.Search({
			placeHolder: 'Search Spaces', 
			name: 'searchfield', 
			flex: 1, 
			listeners: {
				scope: this, 
				keyup: function() {
					this.spaceStore.clearFilter();
					this.spaceStore.filter({
						property: 'name', 
						anyMatch: true, 
						value: this.spaceSearchField.getValue()
					});
				}
			}
		});
		
		var refreshButton = new Ext.Button({
			iconCls: 'refresh',
			ui: 'plain', 
			iconMask: true, 
			handler: this.refresh, 
			scope: this
		});
		
		this.spacePanel = new Ext.form.FormPanel({
			layout: 'fit', 
			bodyPadding: 0, 
			dockedItems: [{
				xtype: 'toolbar',
				ui: 'plain', 
				dock: 'top', 
				layout: 'hbox', 
				items: [this.spaceSearchField, refreshButton], 
			}], 
			listeners: {
				scope: this, 
				show: function() {
					this.toolbar.setTitle('All Spaces');
					this.backButton.hide();
				} 
			}, 
			items: this.spaceList
		});
		
		return this.spacePanel;
	}, 
	
	/**
	 * Creates the page list and search field. 
	 * @returns {Ext.FormPanel}
	 */
	createPagePanel: function() {
		this.pageList = new Ext.List({
			store: this.pageStore, 
			itemTpl: '<div class="page"><div>{title}</div></div>', 
			grouped: true, 
			indexBar: true, 
			listeners: {
				scope: this, 
				itemtap: this.onPageTap
			}
		});
		
		this.pageSearchField = new Ext.form.Search({
			placeHolder: 'Search Pages', 
			name: 'searchfield', 
			flex: 1, 
			listeners: {
				scope: this, 
				keyup: function() {
					this.pageStore.clearFilter();
					this.pageStore.filter({
						property: 'title', 
						anyMatch: true, 
						value: this.pageSearchField.getValue()
					});
				}
			}
		});
		
		var refreshButton = new Ext.Button({
			iconCls: 'refresh',
			ui: 'plain', 
			iconMask: true, 
			handler: this.refresh, 
			scope: this
		});
		
		this.pagePanel = new Ext.form.FormPanel({
			layout: 'fit', 
			bodyPadding: 0, 
			dockedItems: [{
				xtype: 'toolbar',
				ui: 'plain', 
				dock: 'top', 
				layout: 'hbox', 
				items: [this.pageSearchField, refreshButton, {
					xtype: 'button',
					iconCls: 'new-page-icon', 
					height: 32, 
					ui: 'plain', 
					handler: this.createNewPage, 
					scope: this
				}], 
			}], 
			listeners: {
				scope: this, 
				show: function() {
					this.toolbar.setTitle(this.truncateTitle(this.currentSpace.title));
					this.backButton.show();
				}
			}, 
			items: this.pageList
		});
		
		return this.pagePanel;
	}, 
	
	/**
	 * Creates the spaces and pages stores.
	 */
	createStores: function() {
		/**
		 * Holds the list of spaces.
		 * @property
		 */
		this.spaceStore = new Ext.data.Store({
			model: 'app.models.Page', 
			listeners: {
				scope: this, 
				datachanged: function() {
					this.spaceList.scroller.scrollTo({
					    x: 0,
					    y: 0
					});
				}
			},     
			getGroupString : function(record) {
		        return record.get('category')[0];
		    },
		});
		
		/**
		 * Holds all pages in a given space
		 * @property
		 */
		this.pageStore = new Ext.data.Store({
			model: 'app.models.Page', 
			listeners: {
				scope: this, 
				datachanged: function() {
					this.pageList.scroller.scrollTo({
					    x: 0,
					    y: 0
					});
				}
			},     
			getGroupString : function(record) {
		        return record.get('category')[0];
		    },
		});
	}, 
	
	/**
	 * Lists all spaces
	 */
	listSpaces: function() {
		delete this.currentSpace;
		app.confluence.Proxy.listSpaces({
			scope: this, 
			success: function(spaces) {
				for(var i=0; i<spaces.length; i++) {
					/*
					 * Here we create title attribute from name, this is necessary since we
					 * use the same model for spaces and pages
					 */
					var title = spaces[i].name;
					
					Ext.apply(spaces[i], {
						// create case-insensitive index value
						category: title.toUpperCase(),  
						title: title 
					});
				}

				/*
				 * We need to sort by the index property prior to loading the data in the store
				 * to make case-insensitive indexing work properly.  Otherwise, clicking on items in the
				 * last may navigate to unexpected items due to the hidden displacement of lower case
				 * items to the end of the list.
				 */
				this.spaceStore.loadData(app.Utils.sortArray(spaces, 'category', 'ASC'));

				this.layout.setActiveItem(0, {
					type: 'slide',
					direction: 'right'
				});
			}, 
			fail: function(message) {
				Ext.Msg.alert('Error', 'Sorry, spaces cannot be loaded at this time.');
			}
		});
	}, 
	
	/**
	 * Lists all pages in a space
	 * @param {String} spaceKey
	 * @param {String} spaceName
	 */
	listPagesInSpace: function(space) {
		this.currentSpace = space;
		app.confluence.Proxy.listPages({
			spaceKey: space.key, 
			scope: this, 
			success: function(pages) {
				var loadFn = function() {
					this.pageSearchField.setValue('');
					this.pageStore.clearFilter();
					
					// create case-insensitive index value
					for(var i=0; i<pages.length; i++) {
						pages[i].category = pages[i].title.toUpperCase();
					}
					
					/*
					 * We need to sort by the index property prior to loading the data in the store
					 * to make case-insensitive indexing work properly.  Otherwise, clicking on items in the
					 * last may navigate to unexpected items due to the hidden displacement of lower case
					 * items to the end of the list.
					 */
					this.pageStore.loadData(app.Utils.sortArray(pages, 'category', 'ASC'));
				};
				
				if(this.layout.getActiveItem() === this.pagePanel) {
					loadFn.call(this);
				} else {
					this.layout.setActiveItem(1, {
						type: 'slide',
						direction: 'left',
						scope: this, 
						after: loadFn
					});
				}
			}, 
			fail: function(message) {
				Ext.Msg.alert('Error', 'Sorry, pages cannot be loaded at this time.');
			}
		});
	}, 
	
	/**
	 * Switches the card to pages list.
	 * @param list
	 * @param index
	 * @param item
	 * @param event
	 */
	onSpaceTap: function(list, index, item, event) {
		var space = this.spaceStore.getAt(index);
		this.spaceSearchField.blur();
		this.listPagesInSpace(space.data);
	},
	
	/**
	 * Switches the card to the camera form.
	 * @param list
	 * @param index
	 * @param item
	 * @param event
	 */
	onPageTap: function(list, index, item, event) {
		this.currentPage = this.pageStore.getAt(index).data;
		this.pageSearchField.blur();
		this.layout.next({
			type: 'slide',
			direction: 'left'
		});
		this.pageView.loadPage(this.currentPage.id);
	}, 
	
	/**
	 * Refreshes the list currently being viewed.
	 */
	refresh: function() {
		if(this.currentSpace) {
			this.listPagesInSpace(this.currentSpace);
		} else {
			this.listSpaces();
		}
	}, 
	
	showCameraForm: function() {
		this.layout.setActiveItem(3, {
			type: 'slide',
			direction: 'up',
			cover: true, 
			after: function() {
				this.cameraForm.setPage(this.currentPage);
			}, 
			scope: this
		});
	}, 
	
	createNewPage: function() {
		delete this.currentPage;
		this.showCameraForm();
		this.cameraForm.setSpaceKey(this.currentSpace.key);
	}, 
	
	/**
	 * Called when the back button is tapped.
	 */
	goBack: function() {
		var anim = {
			type: 'slide',
			direction: 'right'
		};
		
		if(this.getActiveItem() === this.cameraForm) {
			Ext.apply(anim, {
				reveal: true,
				direction: 'down'
			});
			
			if(!this.currentPage) {
				this.setActiveItem(1, anim);
				return;
			} 
		}
		
		this.layout.prev(anim);
	}
});