Ext.ns('app.xmlrpc');

/**
 * @class app.xmlrpc.XmlRpc
 * A simple implementation of XML RPC. 
 */
app.xmlrpc.XmlRpc = {
	/**
	 * Makes an XML RPC request.
	 * @param {Object} config An object with the following parameters:
	 * 	url - The url to call
	 * 	method - The RPC method to call
	 * 	params - an array of parameters to pass as arguments
	 * 	success - callback function for a successful request 
	 * 	fail - callback function for a failed request 
	 * 	scope - a scope for the callback functions
	 */
	exec: function(config) {
		var mask = new Ext.LoadMask(Ext.getBody(), {
			msg: 'Loading...'
		});
		
		mask.show();
		
		var body = app.xmlrpc.XmlRpc.createBody(config.method, config.params);
		
        var success = function(response, options) {
        	mask.hide();
        	var result;
        	
        	try {
        		result = app.xmlrpc.XmlRpc.parseResponse(response.responseText);
        	} catch(e) {
        		result = {success: false, message: 'Invalid response from server.'};
        	}
        	
        	if(result.success) {
        		config.success.apply(config.scope, result.params);
        	} else {
        		config.fail.call(config.scope, result.message);
        	}
        };
        
        var fail = function(response, options) {
        	mask.hide();
        	config.fail.call(config.scope, message);
        };

        Ext.Ajax.request({
        	url: config.url,
        	scope: this, 
        	success: success, 
        	failure: fail, 
        	params: body, 
        	headers: {
        		'Content-Type':'text/xml'
        	}
        });
        
        //PhoneGap.exec(success, fail, 'EasyRequest', 'curl', [config.url, body]);
	}, 
	
	/**
	 * Creates the XML request body.
	 * @private
	 * @param method
	 * @param params
	 * @returns {Array}
	 */
	createBody: function(method, params) {
		var body = [
        	'<?xml version="1.0"?>',
			'<methodCall>', 
				'<methodName>' + method + '</methodName>', 
				'<params>', 
		];

		for(var i=0; i<params.length; i++) {
			try {
				body.push('<param><value>' + app.xmlrpc.XmlRpc.toXml(params[i]) + '</value></param>');
			} catch(e) {
				alert('error converting params[' + i + '] to xml: ' + e.message);
			}
		}

		body.push('</params>');
		body.push('</methodCall>');
		body = body.join('');
		return body;
	}, 
	
	toXml: function(obj) {
		if(obj instanceof app.xmlrpc.Value) {
			var type = obj.type || 'string';
			return '<' + type + '>' + app.xmlrpc.XmlRpc.escapeXml(obj.value) + '</' + type + '>';
		} else if(Ext.isObject(obj)) {
			var xml = [];
			
			for(key in obj) {
				xml.push('<member><name>' + app.xmlrpc.XmlRpc.escapeXml(key) + '</name><value>' + app.xmlrpc.XmlRpc.toXml(obj[key]) + '</value></member>');
			}
			
			return '<struct>' + xml.join('') + '</struct>';
		} else {
			return '<string>' + app.xmlrpc.XmlRpc.escapeXml(obj) + '</string>';
		}
	}, 
	
	escapeXml: function(str) {
		return str;
	}, 
	
	parseResponse: function(response) {
		var result = {
			success: true, 
			params: []
		};

		var xml = new DOMParser().parseFromString(response,"text/xml");
		var params = xml.firstChild.firstChild;
		
		if(params.tagName == 'fault') {
			result.success = false;
		} else if(params.tagName == 'params') {
			for(var i=0; i<params.childNodes.length; i++) {
				var obj = app.xmlrpc.XmlRpc.parseValueNode(params.childNodes[i].firstChild); // <param> => <value>
				result.params.push(obj);
			}
		} else {
			throw new Error('Unknown tag in response: ' + params.tagName);
		}
		
		return result;
	}, 
	
	parseValueNode: function(node) {
		try {
			var value = node.firstChild.nodeValue;
			var child = node.firstChild;
			var result = null;
			
			if(typeof(value) == 'string') {
				result = value;
			} else if(child.tagName == 'array') {
				result = [];
				var data = child.firstChild;
				
				// recursively parse each value node
				for(var i=0; i<data.childNodes.length; i++) {
					result.push(app.xmlrpc.XmlRpc.parseValueNode(data.childNodes[i]));
				}
			} else if(child.tagName == 'struct') {
				result = {};
				var members = child.childNodes;
				for(var i=0; i<members.length; i++) {
					var memberName = members[i].getElementsByTagName('name')[0].firstChild.nodeValue;
					var memberValue = members[i].getElementsByTagName('value')[0];
					result[memberName] = app.xmlrpc.XmlRpc.parseValueNode(memberValue);
				}
			}
	
			return result;
		} catch(e) {
			console.log('Error in parseValueNode: ' + e.message);
			return {};
		}
	}
};
